package com.redhat.bayesian.lsp;

import java.io.IOException;
import java.util.Arrays;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeoutException;

import org.eclipse.lsp4j.DidChangeTextDocumentParams;
import org.eclipse.lsp4j.InitializeParams;
import org.eclipse.lsp4j.MessageParams;
import org.eclipse.lsp4j.PublishDiagnosticsParams;
import org.eclipse.lsp4j.ShowMessageRequestParams;
import org.eclipse.lsp4j.TextDocumentContentChangeEvent;
import org.eclipse.lsp4j.VersionedTextDocumentIdentifier;
import org.eclipse.lsp4j.jsonrpc.Launcher;
import org.eclipse.lsp4j.launch.LSPLauncher;
import org.eclipse.lsp4j.services.LanguageClient;
import org.eclipse.lsp4j.services.LanguageServer;

public class Main {

    public static void main(String[] args) throws IOException, InterruptedException, ExecutionException, TimeoutException {

        String serverJs = System.getProperty("server.js", "/home/msrb/projects/component-analysis-lsp-server/output/server.js");

        Process process = new ProcessBuilder("node", serverJs, "--stdio").redirectError(ProcessBuilder.Redirect.INHERIT).start();

        LanguageClient client = new LanguageClient() {
            public void telemetryEvent(Object object) {

            }

            public CompletableFuture<Void> showMessageRequest(ShowMessageRequestParams requestParams) {
                return null;
            }

            public void showMessage(MessageParams messageParams) {

            }

            public void publishDiagnostics(PublishDiagnosticsParams diagnostics) {
                if (!diagnostics.getDiagnostics().isEmpty()) {
                    System.out.println(diagnostics.getDiagnostics().get(0).getMessage());
                    System.out.println(diagnostics.getDiagnostics().get(0).getRange());
                } else {
                    System.out.println("No diagnostics...");
                }
            }

            public void logMessage(MessageParams message) {
                System.out.println(message.getMessage());
            }
        };
        Launcher<LanguageServer> launcher = LSPLauncher.createClientLauncher(client, process.getInputStream(), process.getOutputStream());

        launcher.startListening();
        LanguageServer server = launcher.getRemoteProxy();
        server.initialize(new InitializeParams());
        DidChangeTextDocumentParams didChangeTextDocumentParams = new DidChangeTextDocumentParams();
        VersionedTextDocumentIdentifier versionedTextDocumentIdentifier = new VersionedTextDocumentIdentifier();
        versionedTextDocumentIdentifier.setUri("/tmp/pom.xml");
        versionedTextDocumentIdentifier.setVersion(1);
        didChangeTextDocumentParams.setTextDocument(versionedTextDocumentIdentifier);
        TextDocumentContentChangeEvent textDocumentContentChangeEvent = new TextDocumentContentChangeEvent();

        String pom = "<project>" +
                "<groupId>gid</groupId>" +
                "<artifactId>aid</artifactId>" +
                "<version>1</version>" +
                "<dependencies>" +
                "<dependency>" +
                "<groupId>commons-fileupload</groupId>" +
                "<artifactId>commons-fileupload</artifactId>" +
                "<version>1.3</version>" +
                "</dependency>" +
                "</dependencies>" +
                "</project>";

        textDocumentContentChangeEvent.setText(pom);
        didChangeTextDocumentParams.setContentChanges(Arrays.asList(new TextDocumentContentChangeEvent[] { textDocumentContentChangeEvent }));
        server.getTextDocumentService().didChange(didChangeTextDocumentParams);

        server.shutdown();
        process.waitFor();
    }
}
